package org.wonderly.netbeans.perforce;

import org.openide.loaders.DataObject;
import org.openide.cookies.EditorCookie;
import org.openide.nodes.Node;
import org.openide.util.HelpCtx;
import org.openide.util.NbBundle;
import org.openide.util.actions.CookieAction;
import java.util.*;
import java.io.*;
import org.openide.filesystems.*;
import java.util.logging.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import org.netbeans.api.java.project.JavaProjectConstants;
import org.netbeans.api.project.Project;
import org.netbeans.api.project.SourceGroup;
import org.netbeans.api.project.Sources;
import org.netbeans.editor.*;
import org.wonderly.awt.*;
import org.wonderly.swing.*;

/**
 *  This is the base class for Perforce actions.  It knows how to run the command line
 *  collect the output and route it all to the window.
 */
public class PerforceCommand extends CookieAction {
	private static Logger log = Logger.getLogger( PerforceCommand.class.getName() );
	String types[];
	String id;
	String icon;
	boolean ask;

	public PerforceCommand( String actTypes[], String id, String icon ) {
		this( actTypes, id, icon, false );
	}

	public PerforceCommand( String actType, String id, String icon ) {
		this( new String[]{ actType }, id, icon, false );
	}

	public PerforceCommand( String actTypes[], String id, String icon, boolean ask ) {
		types = actTypes;
		this.id = id;
		this.ask = ask;
		this.icon = icon;
	}

	public PerforceCommand( String actType, String id, String icon, boolean ask ) {
		this( new String[]{ actType }, id, icon, ask );
	}

	public static void runInSwing( final Runnable r ) {
		if( SwingUtilities.isEventDispatchThread() ) {
			r.run();
		} else {
			try {
				SwingUtilities.invokeAndWait( r );
			} catch( Exception ex) {
			}
		}
	}

	static void infoText( final String pref, final String str ) {
		log.info( ((pref == null || pref.length() == 0)? "" : pref+":") + str );
		runInSwing( new Runnable() {
			public void run() {
				PerforceOutputTopComponent potc = PerforceOutputTopComponent.findInstance();
				try {
					potc.addOutput(str);
					potc.addOutput("\n");
				} catch( Exception ex ) {
					log.log( Level.SEVERE, ex.toString(), ex );
				}
			}
		});
	}

	static class P4Command {
		ArrayList<String>outl = new ArrayList<String>();
		ArrayList<String>eoutl = new ArrayList<String>();
		Process p;
		Logger log = Logger.getLogger( getClass().getName() );
		Thread outth, errth;
		PerforceOutputTopComponent potc;

		public int waitFor() throws InterruptedException, IOException {
			p.getOutputStream().close();
			int code = p.waitFor();
			log.info("join output thread: "+outth );
			outth.join(4000);
			log.info("join error thread: "+errth );
			errth.join(4000);
			return code;
		}
 
		public Process getProcess() {
			return p;
		}

		private ArrayList<String>l;
		public String getCommand() {
			String ret = "";
			for( int i = 0; l != null && i < l.size(); ++i ) {
				if( i > 0 )
					ret += " ";
				String v = l.get(i);
				if( v.indexOf(' ') > 0 )
					ret += "\""+v+"\"";
				else
					ret += v;
			}
			return ret;
		}
		
		P4Command() {
			runInSwing( new Runnable() {
				public void run() {
					 potc = PerforceOutputTopComponent.findInstance();
				}
			});
		}
		P4Command( String type[], String[] files, InputStreamProvider pr ) {
			this();
			l = new ArrayList<String>();
			l.add("p4");
			for( String st: type ) {
				l.add(st);
			}
			for( String st: files ) {
				l.add(st);
			}
			String[]n = new String[l.size()];
			l.toArray( n );
			runCommandLine( n, pr );
		}

		public P4Command( String type[], Node[]nds ) {
			this();
			l = new ArrayList<String>();
			l.add("p4");
			for( String st: type ) {
				l.add(st);
			}

			for(Node n: nds ) {
				DataObject c = (DataObject) n.getCookie(DataObject.class);
				if(c == null ) {
					Project pc = (Project) n.getCookie(Project.class);
					if( pc == null ) {
						pc = (Project)n.getLookup().lookup( Project.class );
						if( pc == null )
							throw new NullPointerException("No project object available");
					}
					FileObject proj = pc.getProjectDirectory();
					String projdir = "";
					if( proj != null ) {
						if( type[0].equals("edit") ) {
							File f = FileUtil.toFile ( proj );
							try {
								if( f != null ) {
									String str;
									projdir = f.getCanonicalPath();
									
									if( filesUnder(  projdir + File.separator + "nbproject" ).size() > 0 ) {
										l.add( str = projdir + File.separator + "nbproject" + File.separator+"..." );
    									log.info("Adding nbproject path under: "+projdir );
									}
									if( filesUnder(  projdir + File.separator + "build"     ).size() > 0 ) {
										l.add( str = projdir + File.separator + "build"     + File.separator+"..." );
        								log.info("Adding build path under: "+projdir );
									}
								}
							} catch( Exception exx ) {
								reportException(exx);
							}
						} else if( type[0].equals("add") ) {
							try {
								listChildren( proj.getFileObject( "build" ), l );
								listChildren( proj.getFileObject( "nbproject" ), l );
							} catch( Exception exx ) {
								reportException(exx);
							}
							
						} else {
							File f = FileUtil.toFile ( proj );
							try {
								if( f != null ) {
									String str;
									l.add( str = f.getCanonicalPath()+File.separator+"..." );
									projdir = f.getCanonicalPath();
									log.info("Adding project path: "+str);
								}
							} catch( Exception exx ) {
								reportException(exx);
							}
						}
					}
					Sources srcs = (Sources)pc.getLookup().lookup(Sources.class);
					if( srcs == null )
						throw new NullPointerException( "Project: "+n+" has no sources object");
					SourceGroup grps[] = srcs.getSourceGroups(JavaProjectConstants.SOURCES_TYPE_JAVA);;
					for(SourceGroup grp: grps ) {
						FileObject root = grp.getRootFolder();
                        log.info("Checking for files under: "+FileUtil.toFile(root) );
						if( countChildren( root.getChildren() ) > 0 ) {
							File f = FileUtil.toFile ( root );
							log.info("found some children sources under: "+f);
							try {
								if( f != null ) {
//									if( f.getCanonicalPath().startsWith( projdir ) == false ) {
										listChildren( root, l );
//									}
								}
							} catch( Exception exx ) {
								reportException(exx);
							}
						}
					}

//					int chg = PerforceOps.newChangeList();
//					for( String nm: l ) {
//						
//					}
				} else {
					for( FileObject fc : (Set<FileObject>)c.files() ) {
						File f = FileUtil.toFile ( fc );

						try {
							if( f != null ) {
								if( f.isDirectory() ) {
									l.add( f.getCanonicalPath()+"/..." );
								} else {
									l.add( f.getCanonicalPath() );							
								}
							}
						} catch( Exception exx ) {
							reportException(exx);
						}
					}
				}
			}

			String[]n = new String[l.size()];
			l.toArray(n);
			runCommandLine( n, null );
		}
		
		void listChildren( FileObject root, ArrayList<String> list ) throws IOException {
			for( FileObject fo : root.getChildren() ) {
				if( fo.isFolder() == false ) {
					File f = FileUtil.toFile ( fo );
					String str;
					list.add( str = f.getCanonicalPath() );
					log.info("Adding source file: "+str);
				} else {
					listChildren( fo, list );
				}
			}
		}

		java.util.List<String> filesUnder( String dir ) throws IOException {
			System.out.println( "counting files under: " + dir );
			final P4Command pc = new P4Command( new String[]{ "have"}, 
					new String[]{ dir+"\\..." }, null );
			ArrayList<String>cl = new ArrayList<String>();
			int code = -1;
			try {
				System.out.println("Waiting for p4 have "+dir+"\\...");
				code = pc.waitFor();
				log.info("have exits: "+code);
			} catch (Exception ex) {
				log.log( Level.WARNING, ex.toString(), ex );
			}
			if( code != 0 ) {
				throw new IOException( "Perforce opened command failed: "+code);
			}
			log.info("output: "+pc.getOutput()+", len: "+((pc.outl != null) ? pc.outl.size() : 0));
			for( String v : pc.getOutput() ) {
				String str;
				cl.add( str = v.split("#")[0]);
				log.info("have Using File: "+str);
			}
			return cl;
		}

		void runCommandLine( final String[] n, final InputStreamProvider pr ) {
			try {
				String sc = "";
				for( String c : n ) {
					sc += c+" ";
				}

				infoText( null, "starting: "+sc );
				p = Runtime.getRuntime().exec( n);
				InputStream os = p.getInputStream();
				final BufferedReader rd = new BufferedReader( new InputStreamReader(os));
				InputStream es = p.getErrorStream();
				final BufferedReader erd = new BufferedReader( new InputStreamReader(es));
				if( pr != null ) {
					log.info("Configuring for writer: "+pr);
					new Thread() {
						public void run() {
							try {
								pr.writeTo(p.getOutputStream());
								p.getOutputStream().close();
							} catch (IOException ex) {
								log.log( Level.WARNING, ex.toString(), ex );
							}
							log.info("input Stream closed");
						}
					}.start();
				} else {
					log.info("Closing input Stream, no source");
					p.getOutputStream().close();
				}
//				final JTextArea area = potc.getTextArea();
				outth = new Thread() {
					public void run(){
						String str;
						try {
							try {
								log.info("Starting output reading");
								while( (str = rd.readLine()) != null ) {
									infoText("output",str);
									outl.add(str);
									if( str.indexOf( "#") == -1 )
										continue;

									str = str.split("#")[0];
									if( n[1].equals("edit") ) {
										potc.edit(str);
									} else if( n[1].equals("add") ) {
										potc.add( str );
									} else if( n[1].equals("submit") ) {
										potc.submit(str);
									} else if( n[1].equals("delete") ) {
										potc.delete( str );
									} else if( n[1].equals( "revert") ) {
										potc.revert(str);
									}
								}
							} finally {
								rd.close();
							}
							log.info("output eof reached");
						} catch( Exception ex){
							log.log(Level.WARNING,ex.toString(),ex);
						}
					}
				};
				outth.start();

				errth  = new Thread() {
					public void run(){
						String str;
						try {
							try {
								log.info("error Thread starting");
								while( (str = erd.readLine()) != null ) {
									infoText( "error", str );
									eoutl.add(str);
                                    log.info("added error: "+str);
								}
							} finally {
								erd.close();
							}
							log.info("error eof reached");
						} catch( Exception ex) {
							 log.log(Level.WARNING,ex.toString(),ex);
						}
						log.info("Error Thread exiting");
					}
				};
				errth.start();
				log.info(n[0]+" "+n[1]+" processing started");
			 } catch( Exception ex ) {
				log.log(Level.SEVERE,ex.toString(),ex);   
			}
			openWindow();
		}

		private int countChildren( FileObject[] fos ) {
			int cnt = 0;
			for( FileObject f: fos ) {
				if( f.isFolder() ) {
					int cld = countChildren( f.getChildren() );
					log.info( "found " + cld + " children under: " + FileUtil.toFile( f ) );
					cnt += cld;
				} else {
					++cnt;
				}
			}
 			return cnt;
		}

		public java.util.List<String> getOutput() {
			return outl;
		}

		public java.util.List<String> getError() {
			return eoutl;
		}
	}

	public static void openWindow() {
		runInSwing( new Runnable() {
			public void run() {
				PerforceOutputTopComponent.findInstance().open();
				PerforceOutputTopComponent.findInstance().requestActive();
				PerforceOutputTopComponent.findInstance().requestVisible();
			}
		});
	}

	private void showResults( P4Command pc ) {
		java.util.List<String>out = pc.getOutput();
		for( String s: out ) {
			log.info(s);
		}
	}

	protected void performAction( final Node[] activatedNodes ) {
//        System.out.println("dumping node files");
//        for( Node n : activatedNodes ) {
//            DataObject dob = (DataObject)n.getCookie(DataObject.class);
//            Set<FileObject> s = (Set<FileObject>)dob.files();
//            for( FileObject f : s ) {
//                System.out.println(n.getName()+": file "+f.getPath());            
//            }
//        }
		if( ask ) {
			JPanel p = new JPanel();
			Packer pk = new Packer( p );
			int y = -1;
			if( activatedNodes.length < 6 ) {
				pk.pack( new JLabel( "Really " + Arrays.toString(types) )
					).gridx(0).gridy(++y).fillx().inset(6,6,6,6);
				for( Node n : activatedNodes ) {
					DataObject c = (DataObject) n.getCookie(DataObject.class);  
					EditorCookie ec = (EditorCookie) n.getCookie(EditorCookie.class);
					if( ec != null && ec.isModified() ) {
						try {
							ec.saveDocument();
						} catch( Exception ex ) {
							reportException(ex);
						}
					}

					if(c == null ) {
						throw new NullPointerException("can't find FileObject for:"+n);
					}
					for( FileObject fc : (Set<FileObject>)c.files() ) {
						File f = FileUtil.toFile ( fc );
						pk.pack(new JLabel(f.getName())).gridx(0).gridy(++y).inset(2,2,2,2).fillx();
					}
			   }
			} else {
				pk.pack( new JLabel( "Really "+Arrays.toString(types)+" "+activatedNodes.length+
						" files") ).gridx(0).gridy(++y).fillx().inset(5,5,5,5);
			}
			pk.pack( new JSeparator() ).gridx(0).gridy(++y).fillx().inset(6,6,6,6);

			final JButton bs[] = new JButton[3];
			bs[2] = new JButton( NbBundle.getMessage( getClass(), "cancel_button" ) );
			bs[1] = new JButton( NbBundle.getMessage( getClass(), "review_button" ) );
			bs[0] = new JButton( NbBundle.getMessage( getClass(), "okay_button" ) );

			final boolean cancelled[] = new boolean[1];

			bs[2].addActionListener( new ActionListener() {
				public void actionPerformed( ActionEvent ev ) {
					cancelled[0] = true;
				}
			});

			bs[1].addActionListener( new ActionListener() {
				public void actionPerformed( ActionEvent ev ) {
					try {
						infoText("diff", "Starting diff");
						for( Node n : activatedNodes ) {
							new PerforceDiff().performAction( new Node[]{ n } );
						}
					} catch( Exception exx ) {
						reportException(exx);
					}
				}
			});

			bs[0].addActionListener( new ActionListener() {
				public void actionPerformed( ActionEvent ev ) {
					cancelled[0] = false;
				}
			});

			final Dialog dlg[] = new Dialog[1];
			dlg[0] = DialogSupport.createDialog( "Confirm "+Arrays.toString(types)+" operation",
				p, true, bs, false, 2, 2,
				new ActionListener() {
					public void actionPerformed( ActionEvent ev ) {
						if( ev.getSource() != bs[1] ) {
							dlg[0].setVisible(false);
							dlg[0].dispose();
						}
					}
				}
			);

			dlg[0].pack();
			dlg[0].setVisible( true );
			if( cancelled[0] )
				return;
		}

		try {
//            for( Node n: activatedNodes ) {
//                System.out.println("n.getParent(): "+n.getParentNode());
//             
//                    Children cn = n.getChildren();
//                    Node[] cns = cn.getNodes();
//                    for( Node c : cns ) {
//                        System.out.println("Child node: "+c);
//                        
//                    }
//            }
//			final Process pp = p4c.getProcess();
			new ComponentUpdateThread<Object[]>() {
				public Object[] construct() {
					P4Command p4c = new P4Command( types, activatedNodes );
					try {
						return new Object[]{ p4c, new Integer( p4c.waitFor() )};
					} catch( Exception ex ) {
						log.log( Level.SEVERE, ex.toString(), ex );
					}
					return new Object[]{ p4c, new Integer(-1) };
				}
				public void finished() {
					try {
						Object arr[] = getValue();
						P4Command p4c = (P4Command)arr[0];
						int code = ((Integer)arr[1]).intValue();
						if( code != 0 ) {
							showError( p4c, code );
						} else {
							System.out.println("p4 "+Arrays.toString(types)+": exits "+code);
							infoText( "", "p4 "+Arrays.toString(types)+" exited: "+code );
							showResults( p4c );
						}
				   } finally {
						super.finished();
					}
				}
			}.start();
		} catch( Exception exx ) {
			reportException(exx);
		}
	}

	static void showError( P4Command p4c, int code ) {
		JPanel p = new JPanel();
		Packer pk = new Packer( p );
		int y = -1;
		JLabel l;
		pk.pack(l = new JLabel(p4c.getCommand()+" exited: "+code)).gridx(0).gridy(++y).inset(6,6,6,6);
		l.setFont( new Font( "courier",Font.BOLD,16));
		pk.pack( new JSeparator() ).gridx(0).gridy(++y).fillx().inset(5,10,5,10);
		java.util.List<String>strs = p4c.getOutput();
		Font f = new Font( "courier",Font.PLAIN,12);

		for( int i = 0; i < strs.size(); ++i ) {
			pk.pack( l=new JLabel(strs.get(i) ) ).gridx(0).gridy(++y).fillx();
			l.setFont(f);
			if( i + 1 < strs.size() && i > 2 ) {
				pk.pack( l=new JLabel("...")).gridx(0).gridy(++y).fillx().inset(0,5,0,0);
				l.setFont(f);
				break;
			}
		}
		if( strs.size() > 0 ) {
			pk.pack( new JSeparator() ).gridx(0).gridy(++y).fillx().inset(5,5,5,5);
		}
		strs = p4c.getError();
		for( String s : strs ) {
			pk.pack( l=new JLabel(s) ).gridx(0).gridy(++y).fillx().inset(0,5,0,0);
			l.setFont(f);
		}
		pk.pack( new JSeparator() ).gridx(0).gridy(++y).fillx().inset(10,10,10,10);
		final Dialog dlg[] = new Dialog[1];
		final JButton bs[] = new JButton[1];
		bs[0] = new JButton( NbBundle.getMessage( PerforceCommand.class, "okay_button" ) );

		dlg[0] = DialogSupport.createDialog( "Error Performing Perforce Operation",
			p, false, bs, false, 0, 0,
			new ActionListener() {
				public void actionPerformed( ActionEvent ev ) {
					dlg[0].setVisible(false);
					dlg[0].dispose();
				}
			}
		);
		dlg[0].pack();
		dlg[0].setVisible(true);
	}

	protected void initialize() {
		super.initialize();
		// see org.openide.util.actions.SystemAction.iconResource() javadoc for more details
	if( icon == null )
		putValue("noIconInMenu", Boolean.TRUE);
	}

	static void reportException(Throwable exx){
		StringWriter sw = new StringWriter();
		PrintWriter pw = new PrintWriter( sw );
		exx.printStackTrace(pw);
		pw.close();
		infoText( "exception",sw.toString() );
		log.log(Level.SEVERE,exx.toString(),exx);
	}

	protected int mode() {
		return CookieAction.MODE_ANY;
	}

	public String getName() {
		return NbBundle.getMessage(getClass(), id );
	}

	protected Class[] cookieClasses() {
		return new Class[] {
			DataObject.class
		};
	}
	protected String iconResource() {
		return icon == null ? null : "org/wonderly/netbeans/perforce/"+icon;

   }

	public HelpCtx getHelpCtx() {
		return HelpCtx.DEFAULT_HELP;
	}

	protected boolean asynchronous() {
		return false;
	}
}

