package org.wonderly.netbeans.perforce;

import org.openide.loaders.DataObject;
import org.openide.cookies.EditorCookie;
import org.openide.nodes.Node;
import org.openide.util.HelpCtx;
import org.openide.util.NbBundle;
import org.openide.util.actions.CookieAction;
import java.util.*;
import java.io.*;
import org.openide.filesystems.*;
import org.openide.loaders.*;
import java.util.logging.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.event.*;
import org.netbeans.editor.*;
import org.wonderly.awt.*;
import org.wonderly.swing.*;

/**
 *  This class provides the interface to running a "p4merge" to show
 *  visual difference indications.
 */
public class PerforceDiff extends CookieAction {
    private static Logger log = Logger.getLogger( PerforceCommand.class.getName() );
    String types[];
    String id;
    boolean ask;

    public PerforceDiff( String actTypes[], String id ) {
        this( actTypes, id, false );
    }

    public PerforceDiff( String actType, String id ) {
        this( new String[]{ actType }, id, false );
    }

    public PerforceDiff( String actTypes[], String id, boolean ask ) {
        types = actTypes;
        this.id = id;
        this.ask = ask;
    }

    public PerforceDiff( String actType, String id, boolean ask ) {
        this( new String[]{ actType }, id, ask );
    }

    public PerforceDiff() {
        this( "", "CTL_PerforceDiff", false );
    }
    public static void runInSwing( final Runnable r ) {
        if( SwingUtilities.isEventDispatchThread() ) {
            r.run();
        } else {
            try {
                SwingUtilities.invokeAndWait( r );
            } catch( Exception ex) {
            }
        }
    }
    
    protected void initialize() {
        super.initialize();
        // see org.openide.util.actions.SystemAction.iconResource() javadoc for more details
//        putValue("noIconInMenu", Boolean.TRUE);
    }

    static void infoText( final String str ) {
        runInSwing( new Runnable() {
            public void run() {
                PerforceOutputTopComponent potc = PerforceOutputTopComponent.findInstance();
                try {
                    potc.addOutput(str);
                    potc.addOutput("\n");
                } catch( Exception ex ) {
                    log.log( Level.SEVERE, ex.toString(), ex );
                }
            }
        });
    }

    private File fileFor( Node n  ){
        DataObject c = (DataObject) n.getCookie(DataObject.class);
        if(c == null ) {
            throw new NullPointerException("can't find FileObject for:"+n);
        }
        FileObject fc = c.getPrimaryFile();
        File f = FileUtil.toFile ( fc );
        return f;
    }

    private class P4Command {
        ArrayList<String>outl = new ArrayList<String>();
        ArrayList<String>eoutl = new ArrayList<String>();
        Process p;
        Logger log = Logger.getLogger( getClass().getName() );

        public Process getProcess() {
            return p;
        }

        private ArrayList<String>l;
        public String getCommand() {
            String ret = "";
            for( int i = 0; l != null && i < l.size(); ++i ) {
                if( i > 0 )
                    ret += " ";
                String v = l.get(i);
                if( v.indexOf(' ') > 0 )
                    ret += "\""+v+"\"";
                else
                    ret += v;
            }
            return ret;
        }

        public P4Command( String cmd, final String type[], Node[]nds ) {
            l = new ArrayList<String>();
            l.add(cmd);
            for( String st: type ) {
                l.add(st);
            }
            infoText("Command line: "+l);
            for(Node n: nds ) {
//                DataObject c = (DataObject) n.getCookie(DataObject.class);
//                if(c == null ) {
//                    throw new NullPointerException("can't find FileObject for:"+n);
//                }
//                FileObject fc = c.getPrimaryFile();
                File f = fileFor(n);
                try {
                    l.add( f.getCanonicalPath() );
                } catch( Exception exx ) {
                    reportException(exx);
                }
            }

            String[]n = new String[l.size()];
            l.toArray(n);
            try {
                String sc = "";
                for( String c : n ) {
                    sc += c+" ";
                }
                infoText("starting: "+sc );
                p = Runtime.getRuntime().exec( n);
                InputStream os = p.getInputStream();
                final BufferedReader rd = new BufferedReader( new InputStreamReader(os));
                InputStream es = p.getErrorStream();
                final BufferedReader erd = new BufferedReader( new InputStreamReader(es));
                p.getOutputStream().close();
                final PerforceOutputTopComponent potc = PerforceOutputTopComponent.findInstance();

                new Thread() {
                    public void run(){
                        String str;
                        try {
                            try {
                                while( (str = rd.readLine()) != null ) {
                                    outl.add(str);
									if( str.indexOf( "#") == -1 )
										continue;

									str = str.split("#")[0];
									if( type[0].equals("edit") ) {
										potc.edit(str);
									} else if( type[0].equals("add") ) {
										potc.add( str );
									} else if( type[0].equals("submit") ) {
										potc.submit(str);
									} else if( type[0].equals("delete") ) {
										potc.delete( str );
									} else if( type[0].equals( "revert") ) {
										potc.revert(str);
									}
                                }
                            } finally {
                                rd.close();
                            }
                        } catch( Exception ex){
                            log.log(Level.WARNING,ex.toString(),ex);
                        }
                    }
                }.start();

                new Thread() {
                    public void run(){
                        String str;
                        try {
                            try {
                                while( (str = erd.readLine()) != null ) {
                                    infoText( str );
                                    eoutl.add(str);
                                }
                            } finally {
                                erd.close();
                            }
                        } catch( Exception ex){
                             log.log(Level.WARNING,ex.toString(),ex);
                        }
                    }
                }.start();
             } catch( Exception ex ) {
                log.log(Level.SEVERE,ex.toString(),ex);   
            }
            openWindow();
        }
 
        public java.util.List<String> getOutput() {
            return outl;
        }

        public java.util.List<String> getError() {
            return eoutl;
        }
    }

    public static void openWindow() {
        PerforceOutputTopComponent.findInstance().open();
        PerforceOutputTopComponent.findInstance().requestActive();
        PerforceOutputTopComponent.findInstance().requestVisible();
    }

    private void showResults( P4Command pc ) {
        java.util.List<String>out = pc.getOutput();
        for( String s: out ) {
            infoText(s);
        }
    }

    protected void performAction( final Node[] activatedNodes ) {
        final String[]files = new String[2];

        try {
            infoText("Starting diff for: "+Arrays.toString( activatedNodes ) );

			// Start the p4 print command to get the current version from the depot
            final P4Command p4c = new P4Command( "p4", new String[]{ "print", "-q"}, new Node[]{ activatedNodes[0] } );
            final Process pp = p4c.getProcess();
			
			// Use ComponentUpdateThread to have access to a swing thread when we are done.
            new ComponentUpdateThread( ){
                public Object construct() {
                    try {
                        int code = pp.waitFor();
                        System.out.println("diff: exits "+code);
                        infoText( "p4 diff exited: "+code );
                        if( code != 0 ) {
                            showError( p4c, code );
                            return new Boolean(false);
                        }
                        File f = fileFor( activatedNodes[0] );
                        File df = File.createTempFile( "depot-", f.getName() );
                        PrintWriter pw = new PrintWriter( new BufferedWriter( new FileWriter( df ) ) );
                        files[0] = df.toString();
                        files[1] = f.toString();
						// Copy the output into the file to diff with
                        try {
                            for( String s : p4c.getOutput() ){
                                pw.println(s);
                            }
                        } finally {
                            pw.close();
                        }
                    } catch( Exception ex ) {
                        log.log( Level.SEVERE, ex.toString(), ex );
                    }
                    return null;
                }
                public void finished() {
                    try {
                        if( getValue() != null )
                            return;
						// Run the merge command to see the differences
                        final P4Command p4c = new P4Command( "p4merge", files, new Node[]{} );
                        final Process pp = p4c.getProcess();
						
						// At some point we could put a component into here to disable...
                        new ComponentUpdateThread() {
                            public Object construct() {
                                try {
                                    return new Integer( pp.waitFor() );
                                } catch( Exception ex ) {
                                    log.log( Level.SEVERE, ex.toString(), ex );
                                }
                                return new Integer(-1);
                            }
                            public void finished() {
                                try {
                                    int code = ((Integer)getValue()).intValue();
									// Need to do this in a swing thread.
                                    infoText( "p4 "+Arrays.toString(types)+" exited: "+code );
                                    if( code != 0 ) {
                                        showError( p4c, code );
                                    }
                               } finally {
                                    super.finished();
                                }
                            }
                        }.start();
                    } catch( Exception exx ) {
                        reportException(exx);
                    } finally {
                        super.finished();
                    }
                }
            }.start();
        } catch( Exception exx ) {
            reportException(exx);
        }
    }
    
     private void showError( P4Command p4c, int code ) {
        JPanel p = new JPanel();
        Packer pk = new Packer( p );
        int y = -1;
        JLabel l;
        pk.pack(l = new JLabel(p4c.getCommand()+" exited: "+code)).gridx(0).gridy(++y).inset(6,6,6,6);
        l.setFont( new Font( "courier",Font.BOLD,16));
        pk.pack( new JSeparator() ).gridx(0).gridy(++y).fillx().inset(5,10,5,10);
        java.util.List<String>strs = p4c.getOutput();
        Font f = new Font( "courier",Font.PLAIN,12);
        
        for( int i = 0; i < strs.size(); ++i ) {
            pk.pack( l=new JLabel(strs.get(i) ) ).gridx(0).gridy(++y).fillx();
            l.setFont(f);
            if( i + 1 < strs.size() && i > 2 ) {
                pk.pack( l=new JLabel("...")).gridx(0).gridy(++y).fillx().inset(0,5,0,0);
                l.setFont(f);
                break;
            }
        }
        if( strs.size() > 0 ) {
            pk.pack( new JSeparator() ).gridx(0).gridy(++y).fillx().inset(5,5,5,5);
        }
        strs = p4c.getError();
        for( String s : strs ) {
            pk.pack( l=new JLabel(s) ).gridx(0).gridy(++y).fillx().inset(0,5,0,0);
            l.setFont(f);
        }
        pk.pack( new JSeparator() ).gridx(0).gridy(++y).fillx().inset(10,10,10,10);
        final Dialog dlg[] = new Dialog[1];
        final JButton bs[] = new JButton[1];
        bs[0] = new JButton( NbBundle.getMessage( getClass(), "okay_button" ) );
 
        dlg[0] = DialogSupport.createDialog( "Error Performing Visual Diff Operation",
            p, false, bs, false, 0, 0,
            new ActionListener() {
                public void actionPerformed( ActionEvent ev ) {
                    dlg[0].setVisible(false);
                    dlg[0].dispose();
                }
            }
        );
        dlg[0].pack();
        dlg[0].setVisible(true);
    }

    private void reportException(Throwable exx){
        StringWriter sw = new StringWriter();
        PrintWriter pw = new PrintWriter( sw );
        exx.printStackTrace(pw);
        pw.close();
        infoText( sw.toString() );
        log.log(Level.SEVERE,exx.toString(),exx);
    }

    protected int mode() {
        return CookieAction.MODE_ANY;
    }

    public String getName() {
        return NbBundle.getMessage(getClass(), id );
    }

    protected Class[] cookieClasses() {
        return new Class[] {
            DataObject.class
        };
    }
    protected String iconResource() {
        return "org/wonderly/netbeans/perforce/diff.gif";
     
   }

    public HelpCtx getHelpCtx() {
        return HelpCtx.DEFAULT_HELP;
    }

    protected boolean asynchronous() {
        return false;
    }
}
