/*
 * PerforceOps.java
 *
 * Created on April 8, 2006, 11:00 AM
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package org.wonderly.netbeans.perforce;

import java.awt.Dialog;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedReader;
import java.io.EOFException;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.PrintWriter;
import java.io.Reader;
import java.io.StringReader;
import java.io.StringWriter;
import java.util.Arrays;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSeparator;
import javax.swing.JTextArea;
import org.netbeans.editor.DialogSupport;
import org.openide.util.NbBundle;
import org.wonderly.awt.Packer;

/**
 *
 * @author gregg
 */
public class PerforceOps {
	
	/** Creates a new instance of PerforceOps */
	public PerforceOps() {
	}

	public static P4ChangeList newChangeList() throws IOException {
		int cno = -1;
		PerforceProcess p = new PerforceProcess( new String[]{ "p4","change","-o" } );
		if( p.exitCode() == 0 ) {
			PerforceProcess pi = new PerforceProcess( new String[]{ "p4","change","-i" }, 
					new StringReader( p.out.buf.toString()+
						"\n\tExample Description\n\nFiles:\n"), "Description:", true );
			String res = pi.out.buf.toString();
			if( res.startsWith("Change ") && res.endsWith(" created.") ) {
				cno = Integer.parseInt( res.substring( "Change ".length(), res.indexOf( " created.") ).trim() );
			}
		}
		return new P4ChangeList( getUser(), getClient(), cno );
	}

	public static String getUser() throws IOException {
		int cno = -1;
		PerforceProcess p = new PerforceProcess( new String[]{ "p4","user","-o" } );
		if( p.code != 0 )
			throw new IOException("Can't read user with \"p4 user -o\"");
		return readField( "User:", p );
	}

	public static String getClient() throws IOException {
		int cno = -1;
		PerforceProcess p = new PerforceProcess( new String[]{ "p4","client","-o" } );
		if( p.code != 0 )
			throw new IOException("Can't read client with \"p4 client -o\"");
		return readField( "Client:", p );
	}
	
	private static String readField( String fld, PerforceProcess p ) throws IOException {
		BufferedReader rd = new BufferedReader( new StringReader( p.out.buf.toString() ) );
		String v;
		while( ( v = rd.readLine() ) != null ) {
			if( v.startsWith(fld) ) {
				return v.substring(fld.length()).trim();
			}
		}
		throw new EOFException("Can't find field \""+fld+"\" in data stream");
	}

	private static class PerforceProcess {
		String[] cmdline;
		Logger log = Logger.getLogger( getClass().getName() );
		OutputReader out, err;
		int code;

		public int exitCode() {
			return code;
		}

		private void showError( String[] p4c, int code ) {
			JPanel p = new JPanel();
			Packer pk = new Packer( p );
			int y = -1;
			JLabel l;
			pk.pack(l = new JLabel( Arrays.toString(p4c)+" exited: "+code)).gridx(0).gridy(++y).inset(6,6,6,6);
			l.setFont( new Font( "courier",Font.BOLD,16));
			pk.pack( new JSeparator() ).gridx(0).gridy(++y).fillx().inset(5,10,5,10);
			String[]strs = out.buf.toString().split("\n");
			Font f = new Font( "courier",Font.PLAIN,12);

			for( int i = 0; i < strs.length; ++i ) {
				pk.pack( l=new JLabel(strs[i] ) ).gridx(0).gridy(++y).fillx();
				l.setFont(f);
				if( i + 1 < strs.length && i > 2 ) {
					pk.pack( l=new JLabel("...")).gridx(0).gridy(++y).fillx().inset(0,5,0,0);
					l.setFont(f);
					break;
				}
			}

			if( strs.length > 0 ) {
				pk.pack( new JSeparator() ).gridx(0).gridy(++y).fillx().inset(5,5,5,5);
			}
			strs = err.buf.toString().split("\n");
			for( String s : strs ) {
				pk.pack( l=new JLabel(s) ).gridx(0).gridy(++y).fillx().inset(0,5,0,0);
				l.setFont(f);
			}

			pk.pack( new JSeparator() ).gridx(0).gridy(++y).fillx().inset(10,10,10,10);
			final Dialog dlg[] = new Dialog[1];
			final JButton bs[] = new JButton[1];
			bs[0] = new JButton( NbBundle.getMessage( getClass(), "okay_button" ) );

			dlg[0] = DialogSupport.createDialog( "Error Performing Perforce Operation",
				p, false, bs, false, 0, 0,
				new ActionListener() {
					public void actionPerformed( ActionEvent ev ) {
						dlg[0].setVisible(false);
						dlg[0].dispose();
					}
				}
			);
			dlg[0].pack();
			dlg[0].setVisible(true);
		}
		
		private void showDialog( JPanel p ) {
			final Dialog dlg[] = new Dialog[1];
			final JButton bs[] = new JButton[1];
			bs[0] = new JButton( NbBundle.getMessage( getClass(), "okay_button" ) );

			dlg[0] = DialogSupport.createDialog( "Error Performing Perforce Operation",
				p, false, bs, false, 0, 0,
				new ActionListener() {
					public void actionPerformed( ActionEvent ev ) {
						dlg[0].setVisible(false);
						dlg[0].dispose();
					}
				}
			);
			dlg[0].pack();
			dlg[0].setVisible(true);
		}

	    private void reportException(Throwable exx){
			StringWriter sw = new StringWriter();
			PrintWriter pw = new PrintWriter( sw );
			exx.printStackTrace(pw);
			pw.close();
			JPanel p = new JPanel();
			Packer pk = new Packer(p);
			JTextArea area;
			pk.pack( new JScrollPane( area = new JTextArea( sw.toString() ) ) ).fillboth();
			area.setEditable(false);
			showDialog( p );

			log.log(Level.SEVERE,exx.toString(),exx);
		}

		public PerforceProcess( String cmd[] ) {
			cmdline = cmd;
			try {
				Process p = Runtime.getRuntime().exec( cmdline );
				out = new OutputReader( p.getInputStream() );
				err = new OutputReader( p.getErrorStream() );
				p.getOutputStream().close();
				code = p.waitFor();
				if( code != 0 ) {
					showError( cmd, code );
				}
			} catch ( Exception ex) {
				reportException( ex );
				code = -1;
			}
		}

		public PerforceProcess( String cmd[], Reader rd, String stopat, boolean after ) {
			cmdline = cmd;
			try {
				Process p = Runtime.getRuntime().exec( cmdline );
				out = new OutputReader( p.getInputStream() );
				err = new OutputReader( p.getErrorStream() );
				InputWriter wr = new InputWriter( rd, p.getOutputStream(), stopat, after );
				new Thread(wr).start();
				code = p.waitFor();
				if( code != 0 ) {
					showError( cmd, code );
				} else if( wr.code != 0 ) {
					showError( cmd, code );
				}
			} catch ( Exception ex) {
				reportException( ex );
				code = -1;
			}
		}
		
		private class InputWriter implements Runnable {
			Reader rd;
			OutputStream os;
			String stopat;
			boolean after;
			int code;
	
			public InputWriter( Reader rd, OutputStream os, String stopat, boolean after ) {
				this.rd = rd;
				this.os = os;
				this.stopat = stopat;
				this.after = after;
			}
			
			public void run() {
				try {
					BufferedReader br = new BufferedReader( rd );
					String str;
					while( ( str = br.readLine() ) != null ) {
						if( str.startsWith( stopat ) && !after )
							break;
						os.write( str.getBytes() );
						os.write("\r\n".getBytes());
						if( str.startsWith( stopat ) )
							break;
					}
					rd.close();
					code = 0;
				} catch( Exception ex ) {
					reportException(ex);
					code = -1;
				} finally {
					try {
						os.close();
					} catch( IOException ex ) {
						log.log( Level.FINE, ex.toString(), ex );
					}
				}
			}
		}

		private class OutputReader implements Runnable {
			StringBuffer buf;
			BufferedReader rd;
			public OutputReader( InputStream in ) {
				rd = new BufferedReader( new InputStreamReader( in ) );
				buf = new StringBuffer();
				new Thread( this ).start();
			}
			public void run() {
				try {
					String res;
					while( ( res = rd.readLine() ) != null ) {
						buf.append(res);
						buf.append("\n");
					}
				} catch( Exception ex ) {
					log.log( Level.WARNING, ex.toString(), ex );
				}
			}
		}
	}
}
