package org.wonderly.netbeans.perforce;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.Serializable;
import java.util.Collections;
import java.util.Comparator;
import java.util.Vector;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JComponent;
import javax.swing.JRadioButtonMenuItem;
import javax.swing.JSplitPane;
import javax.swing.SwingUtilities;
import org.openide.ErrorManager;
import org.openide.util.NbBundle;
import org.openide.util.Utilities;
import org.openide.windows.TopComponent;
import org.openide.windows.WindowManager;
import org.wonderly.awt.Packer;
import org.wonderly.swing.VectorListModel;

/**
 * Top component which displays something.
 */
final class PerforceOutputTopComponent extends TopComponent {

    private static final long serialVersionUID = 1L;
	VectorListModel<String> histmod;

	Logger log = Logger.getLogger( getClass().getName() );
    private static PerforceOutputTopComponent instance;
    /** path to the icon used by the component and its open action */
    static final String ICON_PATH = "nbresloc:/org/wonderly/netbeans/perforce/p4.gif";

    private static final String PREFERRED_ID = "PerforceOutputTopComponent";

    private PerforceOutputTopComponent() {
        initComponents();
		this.
//		mainpanel.remove(splitpane);
//		Packer pk = new Packer( mainpanel );
//		pk.pack( outScroller ).fillboth();
        setName(NbBundle.getMessage(PerforceOutputTopComponent.class, "CTL_PerforceOutputTopComponent"));
        setToolTipText(NbBundle.getMessage(PerforceOutputTopComponent.class, "HINT_PerforceOutputTopComponent"));
        setIcon(Utilities.loadImage(ICON_PATH, true));
    }
	
	public void edit( final String name ) {
		runInSwing( new Runnable() {
			public void run() {
				sort("="+name);
				log.fine( "edit size: "+histmod.size() );
			}
		});
	}
	
	public void add( final String name ) {
		runInSwing( new Runnable() {
			public void run() {
				sort("+"+name);				
				log.fine( "add size: "+histmod.size() );
			}
		});
	}

	public void delete( final String name ) {
		runInSwing( new Runnable() {
			public void run() {
				sort("-"+name);				
				log.fine( "delete size: "+histmod.size() );
			}
		});
	}

	public void submit( final String name ) {
		System.out.println("submit: "+name );
		runInSwing( new Runnable() {
			public void run() {
				histmod.removeElement( "+"+name );
				histmod.removeElement( "-"+name );
				histmod.removeElement( "="+name );
				sort();
			}
		});
	}

	public void revert( final String name ) {
		System.out.println("revert: "+name);
		runInSwing( new Runnable() {
			public void run() {
				histmod.removeElement( "+"+name );
				histmod.removeElement( "-"+name );
				histmod.removeElement( "="+name );
				sort();
			}
		});
	}
	
	private void runInSwing( final Runnable r ) {
		if( SwingUtilities.isEventDispatchThread() ) {
			r.run();
		} else {
			try {
				SwingUtilities.invokeAndWait( r );
			} catch( Exception ex ) {
			}
		}
	}

	private void sort( String val ) {
		Level olevel = log.getLevel();
		log.setLevel( Level.FINE );
		log.fine("sorting: "+val);
		if( histmod.contains( val ) == false ) {
			log.fine( "after contains size: "+histmod.size() );
			log.fine("adding: "+val);
			Vector<String> v = new Vector<String>();
			for( String n : histmod ) {
				v.add(n);
			}
			v.add( val );
			log.fine( "after add size: "+histmod.size() );
			histmod.setContents( v );
			sort();
			histout.repaint();
			log.fine( "after paint size: "+histmod.size() );
			for( String s: histmod ) {
				log.fine("hist: "+s);
			}
		}
		log.setLevel( olevel );
	}

	private void sort() {
		log.fine( "before sort size: "+histmod.size() );
		Collections.sort( histmod, new Comparator() {
			public int compare( Object o1, Object o2 ) {
				return o1.toString().compareTo( o2.toString() );
			}
		});
		histout.setModel( histmod );
		log.fine( "after sort size: "+histmod.size() );
		histout.repaint();
	}

	public void addOutput( final String str ) {
		log.fine(str);
		runInSwing( new Runnable() {
			public void run() {
				outputArea.append( str );
				outputArea.setCaretPosition( outputArea.getDocument().getLength() );
				outScroller.getVerticalScrollBar().setValue( outScroller.getVerticalScrollBar().getMaximum() );
				outputArea.repaint();
			}
		});
	}

//    public javax.swing.JTextArea getTextArea() {
//        return outputArea;
//    }

    private void initComponents() {
        popupMenu = new javax.swing.JPopupMenu();

        splitHori = new JRadioButtonMenuItem("Split Horizontal");
        splitHori.setSelected(true);
        splitVert = new JRadioButtonMenuItem("Split Vertical");

        jSeparator1 = new javax.swing.JSeparator();
        showList = new JCheckBoxMenuItem("Show File List");
        showMessages = new JCheckBoxMenuItem("Show Perforce Messages");

        splitBtnGroup = new javax.swing.ButtonGroup();
        mainpanel = new javax.swing.JPanel();
        splitpane = new javax.swing.JSplitPane();
        histPanel = new javax.swing.JPanel();
        histout = new javax.swing.JList( histmod = new VectorListModel<String>() );
        histScroller = new javax.swing.JScrollPane( histout );
        outPanel = new javax.swing.JPanel();
        outputArea = new javax.swing.JTextArea();
        outScroller = new javax.swing.JScrollPane(outputArea);

		splitHori.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent ev ) {
				splitHoriActionPerformed( ev );
			}
		});

		splitVert.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent ev ) {
				splitVertActionPerformed( ev );
			}
		});

		showList.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent ev ) {
				showListActionPerformed( ev );
			}
		});

		showMessages.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent ev ) {
				showMessagesActionPerformed( ev );
			}
		});

        popupMenu.add(splitHori);
        popupMenu.add(splitVert);
        popupMenu.add(jSeparator1);
        popupMenu.add(showList);
        popupMenu.add(showMessages);

        splitBtnGroup.add( splitHori );
        splitBtnGroup.add( splitVert );

        splitpane.setDividerLocation(100);
		if( splitHori.isSelected() == false && splitVert.isSelected() == false )
			splitHori.setSelected( true );
        splitpane.setOrientation( splitHori.isSelected() ?
            JSplitPane.HORIZONTAL_SPLIT : JSplitPane.VERTICAL_SPLIT );
        splitpane.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mousePressed(java.awt.event.MouseEvent evt) {
                popupHandler(evt);
            }
            public void mouseReleased(java.awt.event.MouseEvent evt) {
                popupHandler(evt);
            }
        });

        histPanel.setBorder(javax.swing.BorderFactory.createTitledBorder("Depot File List"));
        histout.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mousePressed(java.awt.event.MouseEvent evt) {
                popupHandler(evt);
            }
            public void mouseReleased(java.awt.event.MouseEvent evt) {
                popupHandler(evt);
            }
        });
        Packer jpk2 = new Packer( histPanel );
        jpk2.pack( histScroller ).fillboth();

        outPanel.setBorder(javax.swing.BorderFactory.createTitledBorder("Perforce Output"));
        outputArea.setColumns(20);
        outputArea.setRows(5);
        outputArea.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mousePressed(java.awt.event.MouseEvent evt) {
                popupHandler(evt);
            }
            public void mouseReleased(java.awt.event.MouseEvent evt) {
                popupHandler(evt);
            }
        });
        Packer jpk1 = new Packer( outPanel );
        jpk1.pack( outScroller ).fillboth();

        splitpane.setLeftComponent(histPanel);
        splitpane.setRightComponent(outPanel);
        
        Packer pk = new Packer( this );
        pk.pack( mainpanel ).fillboth();
		setupPane();
    }

	private void splitVertActionPerformed(java.awt.event.ActionEvent evt) {
		this.firePropertyChange( "splitVertical", !splitVert.isSelected(), splitVert.isSelected() );
		splitpane.setOrientation( JSplitPane.VERTICAL_SPLIT );
		splitpane.setDividerLocation(.5);
		splitpane.repaint();
// TODO add your handling code here:
	}

	private void splitHoriActionPerformed(java.awt.event.ActionEvent evt) {
		this.firePropertyChange( "splitVertical", !splitVert.isSelected(), splitVert.isSelected() );
		splitpane.setOrientation( JSplitPane.HORIZONTAL_SPLIT );
		splitpane.setDividerLocation(.5);
		splitpane.repaint();
// TODO add your handling code here:
	}

	private void setupPane() {

		boolean isVisMsgs = showMessages.isSelected();
		boolean isVisHist = showList.isSelected();
		
		if( !isVisMsgs && !isVisHist ) {
			showMessages.setSelected( isVisMsgs = true );
			showList.setSelected( isVisHist = true );
		}

		if( isVisHist && isVisMsgs ) {
			splitHori.setEnabled(true);
			splitVert.setEnabled(true);
			showList.setEnabled(true);
			showMessages.setEnabled(true);
			Packer pk = new Packer( mainpanel );
			mainpanel.remove( outPanel );
			mainpanel.remove( histPanel );
			pk.pack( splitpane ).fillboth();
			splitpane.setLeftComponent(histPanel);
			splitpane.setRightComponent(outPanel);
			mainpanel.revalidate();
			mainpanel.repaint();
		} else if( isVisHist ) {
			splitHori.setEnabled(false);
			splitVert.setEnabled(false);
			mainpanel.remove( splitpane );
			Packer pk = new Packer( mainpanel );
			pk.pack( histPanel ).fillboth();
			showList.setEnabled(false);
			mainpanel.revalidate();
			mainpanel.repaint();
		} else if( isVisMsgs ) {
			splitHori.setEnabled(false);
			splitVert.setEnabled(false);
			mainpanel.remove( splitpane );
			Packer pk = new Packer( mainpanel );
			pk.pack( outPanel ).fillboth();
			showMessages.setEnabled(false);
			mainpanel.revalidate();
			mainpanel.repaint();
		} else {
			log.warning("unexpected option configuration for visible panels");
		}
	}

	private void showListActionPerformed(java.awt.event.ActionEvent evt) {
		setupPane();
		this.firePropertyChange( "showHistory", !showList.isSelected(), showList.isSelected() );
	}

	private void showMessagesActionPerformed(java.awt.event.ActionEvent evt) {
		setupPane();
		this.firePropertyChange( "showMessages", !showMessages.isSelected(), showMessages.isSelected() );
	}

	private void popupHandler(java.awt.event.MouseEvent evt) {
		if( evt.isPopupTrigger() ) {
			popupMenu.show( (JComponent)evt.getSource(), evt.getX(), evt.getY() );
		}
	}
    
    // Variables declaration - do not modify
    private javax.swing.JScrollPane histScroller;
    private javax.swing.JList histout;
    private javax.swing.JPanel outPanel;
    private javax.swing.JPanel histPanel;
    private javax.swing.JSeparator jSeparator1;
    private javax.swing.JPanel mainpanel;
    private javax.swing.JScrollPane outScroller;
    private javax.swing.JTextArea outputArea;
    private javax.swing.JPopupMenu popupMenu;
    private javax.swing.JCheckBoxMenuItem showList;
    private javax.swing.JCheckBoxMenuItem showMessages;
    private javax.swing.ButtonGroup splitBtnGroup;
    private javax.swing.JRadioButtonMenuItem splitHori;
    private javax.swing.JRadioButtonMenuItem splitVert;
    private javax.swing.JSplitPane splitpane;
    // End of variables declaration
    
    /**
     * Gets default instance. Do not use directly: reserved for *.settings files only,
     * i.e. deserialization routines; otherwise you could get a non-deserialized instance.
     * To obtain the singleton instance, use {@link findInstance}.
     */
    public static synchronized PerforceOutputTopComponent getDefault() {
        if (instance == null) {
            instance = new PerforceOutputTopComponent();
        }
        return instance;
    }
    
    /**
     * Obtain the PerforceOutputTopComponent instance. Never call {@link #getDefault} directly!
     */
    public static synchronized PerforceOutputTopComponent findInstance() {
        TopComponent win = WindowManager.getDefault().findTopComponent(PREFERRED_ID);
        if (win == null) {
            ErrorManager.getDefault().log(ErrorManager.WARNING, "Cannot find PerforceOutput component. It will not be located properly in the window system.");
            return getDefault();
        }
        if (win instanceof PerforceOutputTopComponent) {
            return (PerforceOutputTopComponent)win;
        }
        ErrorManager.getDefault().log(ErrorManager.WARNING, "There seem to be multiple components with the '" + PREFERRED_ID + "' ID. That is a potential source of errors and unexpected behavior.");
        return getDefault();
    }
    
    public int getPersistenceType() {
        return TopComponent.PERSISTENCE_ALWAYS;
    }
    
    public void componentOpened() {
        // TODO add custom code on component opening
    }
    
    public void componentClosed() {
        // TODO add custom code on component closing
    }
    
    /** replaces this in object stream */
    public Object writeReplace() {
        return new ResolvableHelper();
    }
    
    protected String preferredID() {
        return PREFERRED_ID;
    }
    
    final static class ResolvableHelper implements Serializable {
        private static final long serialVersionUID = 1L;
        public Object readResolve() {
            return PerforceOutputTopComponent.getDefault();
        }
    }
}
