package org.wonderly.netbeans.perforce;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.netbeans.api.java.project.JavaProjectConstants;
import org.netbeans.api.project.Project;
import org.netbeans.api.project.SourceGroup;
import org.netbeans.api.project.Sources;
import org.openide.filesystems.FileObject;
import org.openide.filesystems.FileUtil;
import org.openide.nodes.Node;
import org.openide.util.actions.CookieAction;
import org.wonderly.netbeans.perforce.PerforceCommand.P4Command;
import org.wonderly.swing.ComponentUpdateThread;

public final class SubmitP4Project extends PerforceCommand {
	public SubmitP4Project() {
		super( "submit", "CTL_SubmitP4Project", "psubmit.gif");
	}

	static Logger log = Logger.getLogger( SubmitP4Project.class.getName() );
  
	protected void performAction(Node[] activatedNodes) {
		Project c = (Project) activatedNodes[0].getLookup().lookup(Project.class);

		System.out.println("Getting new changelist for project: "+c);
		P4ChangeList rcl = null;
		try {
			rcl = PerforceOps.newChangeList();
		} catch( IOException ex) {
			reportException(ex);
			return;
		}
		final P4ChangeList cl = rcl;

		System.out.println("Got new changelist: "+cl);

		Sources sc = (Sources)c.getLookup().lookup( Sources.class );
		SourceGroup[] grps = sc.getSourceGroups( JavaProjectConstants.SOURCES_TYPE_JAVA );
		ArrayList<String> l = new ArrayList<String>();
		for(SourceGroup grp: grps ) {
			FileObject root = grp.getRootFolder();
			if( countChildren( root.getChildren() ) > 0 ) {
				File f = FileUtil.toFile ( root );

				try {
					if( f != null ) {
						String str;
						l.add( str = f.getCanonicalPath()+File.separator+"..." );
						log.info("Adding source path: "+str);
					}
				} catch( Exception exx ) {
					reportException(exx);
					return;
				}
			}
		}

		String[]n = new String[l.size()];
		l.toArray(n);
		log.info("running opened to get the files actually being edited: "+l);
		final P4Command pc = new P4Command( new String[]{ "opened"}, n, null );

		new ComponentUpdateThread() {
			public Object construct() {
				int code = -1;
				try {
					code = pc.waitFor();
					log.info("opened exits: "+code);
				} catch (Exception ex) {
					log.log( Level.FINE, ex.toString(), ex );
				}
				if( code != 0 ) {
					reportException( new IOException( "Perforce opened command failed: "+code));
				}
				log.info("output: "+pc.getOutput()+", len: "+((pc.outl != null) ? pc.outl.size() : 0));
				for( String v : pc.getOutput() ) {
					String str;
					cl.add( str = v.split("#")[0]);
					System.out.println("Using File: "+str);
				}

				try {
					cl.writeTo( System.out );
				} catch( Exception ex ) {
					log.log( Level.WARNING, ex.toString(), ex );
				}
				if( cl.prepareSubmit() )
					new P4Command( new String[]{ "submit"}, new String[]{"-i"}, cl );
				return null;
			}
		}.start();
	}

	private int countChildren( FileObject[] fos ) {
		int cnt = 0;
		for( FileObject f: fos ) {
			if( f.isFolder() ) {
				cnt += countChildren( f.getChildren() );
			} else {
				++cnt;
			}
		}
		return cnt;
	}

	protected int mode() {
		return CookieAction.MODE_ALL;
	}

//	public String getName() {
//		return NbBundle.getMessage(SubmitP4Project.class, "CTL_SubmitP4Project");
//	}
	
	protected Class[] cookieClasses() {
		return new Class[] {
			Project.class
		};
	}
	
	protected String iconResource() {
		return "org/wonderly/netbeans/perforce/psubmit.gif";
	}
//	
//	public HelpCtx getHelpCtx() {
//		return HelpCtx.DEFAULT_HELP;
//	}
//	
//	protected boolean asynchronous() {
//		return false;
//	}
	
}

